@@ -1,22 +1,21 @@
-#ifndef lint
-static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
-#endif
-
 #include <stdlib.h>
-
+#ifndef lint
+#if 0
+static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
+#else
+#if defined(__NetBSD__) && defined(__IDSTRING)
+__IDSTRING(yyrcsid, "$NetBSD: skeleton.c,v 1.26 2006/05/24 18:01:43 christos Exp $");
+#endif /* __NetBSD__ && __IDSTRING */
+#endif /* 0 */
+#endif /* lint */
 #define YYBYACC 1
 #define YYMAJOR 1
 #define YYMINOR 9
-#define YYPATCH 20050813
-
-#define YYEMPTY (-1)
-#define yyclearin    (yychar = YYEMPTY)
-#define yyerrok      (yyerrflag = 0)
-#define YYRECOVERING (yyerrflag != 0)
-
-extern int yyparse(void);
-
-static int yygrowstack(void);
+#define YYLEX yylex()
+#define YYEMPTY -1
+#define yyclearin (yychar=(YYEMPTY))
+#define yyerrok (yyerrflag=0)
+#define YYRECOVERING (yyerrflag!=0)
 #define YYPREFIX "yy"
 #line 36 "parser.y"
 #ifdef HAVE_DMX_CONFIG_H
@@ -54,7 +53,7 @@ typedef union {
     DMXConfigVirtualPtr    virtual;
     DMXConfigEntryPtr      entry;
 } YYSTYPE;
-#line 58 "parser.c"
+#line 57 "parser.c"
 #define T_VIRTUAL 257
 #define T_DISPLAY 258
 #define T_WALL 259
@@ -67,7 +66,7 @@ typedef union {
 #define T_COMMENT 266
 #define T_LINE_COMMENT 267
 #define YYERRCODE 256
-short yylhs[] = {                                        -1,
+const short yylhs[] = {                                        -1,
     0,   22,   22,   23,   23,   21,   21,   21,   21,   18,
    18,   19,   19,   19,   19,   19,   14,   15,   15,   16,
    16,   17,   11,   11,   11,   12,   12,   12,   13,   13,
@@ -75,7 +74,7 @@ short yylhs[] = {                                        -1,
     7,    7,    8,    8,    9,    9,   10,   10,    3,    3,
     4,    4,    5,    5,    2,    2,    6,    6,
 };
-short yylen[] = {                                         2,
+const short yylen[] = {                                         2,
     1,    1,    2,    1,    1,    4,    5,    5,    6,    1,
     2,    1,    1,    1,    1,    1,    3,    3,    4,    1,
     2,    2,    2,    1,    1,    3,    2,    1,    5,    4,
@@ -83,7 +82,7 @@ short yylen[] = {                                         2,
     1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
     1,    2,    1,    2,    1,    2,    1,    2,
 };
-short yydefred[] = {                                      0,
+const short yydefred[] = {                                      0,
     0,    5,    0,    4,    0,    2,    0,    0,    0,    0,
     0,    0,    3,   52,   42,   44,    0,    0,    0,    0,
    12,    0,    0,   13,   15,   16,    0,   10,   14,    0,
@@ -95,12 +94,12 @@ short yydefred[] = {                                      0,
    19,   21,   48,   31,   32,    0,   26,   30,   37,    0,
     9,   29,   36,
 };
-short yydgoto[] = {                                       3,
+const short yydgoto[] = {                                       3,
    22,   23,   42,   10,   51,   58,   36,   44,   45,   67,
    46,   47,   24,   25,   26,   59,   60,   27,   28,   29,
     4,    5,    6,
 };
-short yysindex[] = {                                   -224,
+const short yysindex[] = {                                   -224,
  -117,    0,    0,    0, -224,    0, -238, -235, -227, -155,
  -109,  -79,    0,    0,    0,    0, -216, -212, -203, -101,
     0,  -34, -198,    0,    0,    0, -110,    0,    0, -155,
@@ -112,7 +111,7 @@ short yysindex[] = {                                   -224,
     0,    0,    0,    0,    0,   32,    0,    0,    0,  -49,
     0,    0,    0,
 };
-short yyrindex[] = {                                      0,
+const short yyrindex[] = {                                      0,
     0,    0,    0,    0,   99,    0, -165,  -47,  -43,    0,
     0,    0,    0,    0,    0,    0,  -30, -175,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
@@ -124,13 +123,13 @@ short yyrindex[] = {                                      0,
     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,
 };
-short yygindex[] = {                                      0,
+const short yygindex[] = {                                      0,
     0,    0,   -1,   70,   -4,    4,   10,   29,   64,   -6,
   -14,   66,    0,    0,    0,    0,   51,    6,    8,    0,
     0,    0,  106,
 };
 #define YYTABLESIZE 268
-short yytable[] = {                                      41,
+const short yytable[] = {                                      41,
    53,   39,   28,   43,   49,    7,   40,   45,   24,   39,
    11,   41,   40,    7,   50,   43,   39,   50,   39,   45,
    24,    7,   35,   38,   39,   63,   48,   14,   39,   12,
@@ -159,7 +158,7 @@ short yytable[] = {                                      41,
     0,    0,    0,    0,    0,    0,    0,   53,   53,   53,
    53,   53,    0,    0,    0,    0,    0,   53,
 };
-short yycheck[] = {                                      47,
+const short yycheck[] = {                                      47,
     0,   59,   59,   47,  125,  123,   47,   47,   47,   59,
     1,   59,   47,  123,  125,   59,   47,  125,   59,   59,
    59,  123,   19,   20,   59,   40,   23,  266,   59,    1,
@@ -194,7 +193,7 @@ short yycheck[] = {                                      47,
 #endif
 #define YYMAXTOKEN 267
 #if YYDEBUG
-char *yyname[] = {
+const char * const yyname[] = {
 "end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,"';'",0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -205,7 +204,7 @@ char *yyname[] = {
 "T_DISPLAY","T_WALL","T_OPTION","T_PARAM","T_STRING","T_DIMENSION","T_OFFSET",
 "T_ORIGIN","T_COMMENT","T_LINE_COMMENT",
 };
-char *yyrule[] = {
+const char * const yyrule[] = {
 "$accept : Program",
 "Program : EntryList",
 "EntryList : Entry",
@@ -267,40 +266,33 @@ char *yyrule[] = {
 "NameList : NameList Name",
 };
 #endif
-#if YYDEBUG
-#include <stdio.h>
-#endif
-
-/* define the initial stack-sizes */
 #ifdef YYSTACKSIZE
 #undef YYMAXDEPTH
-#define YYMAXDEPTH  YYSTACKSIZE
+#define YYMAXDEPTH YYSTACKSIZE
 #else
 #ifdef YYMAXDEPTH
 #define YYSTACKSIZE YYMAXDEPTH
 #else
-#define YYSTACKSIZE 500
-#define YYMAXDEPTH  500
+#define YYSTACKSIZE 10000
+#define YYMAXDEPTH 10000
 #endif
 #endif
-
-#define YYINITSTACKSIZE 500
-
-int      yydebug;
-int      yynerrs;
-int      yyerrflag;
-int      yychar;
-short   *yyssp;
+#define YYINITSTACKSIZE 200
+int yydebug;
+int yynerrs;
+int yyerrflag;
+int yychar;
+short *yyssp;
 YYSTYPE *yyvsp;
-YYSTYPE  yyval;
-YYSTYPE  yylval;
-
-/* variables for the parser stack */
-static short   *yyss;
-static short   *yysslim;
-static YYSTYPE *yyvs;
-static int      yystacksize;
+YYSTYPE yyval;
+YYSTYPE yylval;
+short *yyss;
+short *yysslim;
+YYSTYPE *yyvs;
+int yystacksize;
+int yyparse(void);
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
+static int yygrowstack(void);
 static int yygrowstack(void)
 {
     int newsize, i;
@@ -313,22 +305,13 @@ static int yygrowstack(void)
         return -1;
     else if ((newsize *= 2) > YYMAXDEPTH)
         newsize = YYMAXDEPTH;
-
     i = yyssp - yyss;
-    newss = (yyss != 0)
-          ? (short *)realloc(yyss, newsize * sizeof(*newss))
-          : (short *)malloc(newsize * sizeof(*newss));
-    if (newss == 0)
+    if ((newss = (short *)realloc(yyss, newsize * sizeof *newss)) == NULL)
         return -1;
-
-    yyss  = newss;
+    yyss = newss;
     yyssp = newss + i;
-    newvs = (yyvs != 0)
-          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
-          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
-    if (newvs == 0)
+    if ((newvs = (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs)) == NULL)
         return -1;
-
     yyvs = newvs;
     yyvsp = newvs + i;
     yystacksize = newsize;
@@ -343,11 +326,11 @@ static int yygrowstack(void)
 int
 yyparse(void)
 {
-    register int yym, yyn, yystate;
+    int yym, yyn, yystate;
 #if YYDEBUG
-    register const char *yys;
+    const char *yys;
 
-    if ((yys = getenv("YYDEBUG")) != 0)
+    if ((yys = getenv("YYDEBUG")) != NULL)
     {
         yyn = *yys;
         if (yyn >= '0' && yyn <= '9')
@@ -357,7 +340,7 @@ yyparse(void)
 
     yynerrs = 0;
     yyerrflag = 0;
-    yychar = YYEMPTY;
+    yychar = (-1);
 
     if (yyss == NULL && yygrowstack()) goto yyoverflow;
     yyssp = yyss;
@@ -394,7 +377,7 @@ yyloop:
         }
         *++yyssp = yystate = yytable[yyn];
         *++yyvsp = yylval;
-        yychar = YYEMPTY;
+        yychar = (-1);
         if (yyerrflag > 0)  --yyerrflag;
         goto yyloop;
     }
@@ -405,16 +388,12 @@ yyloop:
         goto yyreduce;
     }
     if (yyerrflag) goto yyinrecovery;
-
+    goto yynewerror;
+yynewerror:
     yyerror("syntax error");
-
-#ifdef lint
     goto yyerrlab;
-#endif
-
 yyerrlab:
     ++yynerrs;
-
 yyinrecovery:
     if (yyerrflag < 3)
     {
@@ -463,10 +442,9 @@ yyinrecovery:
                     YYPREFIX, yystate, yychar, yys);
         }
 #endif
-        yychar = YYEMPTY;
+        yychar = (-1);
         goto yyloop;
     }
-
 yyreduce:
 #if YYDEBUG
     if (yydebug)
@@ -659,7 +637,7 @@ case 58:
 #line 225 "parser.y"
 { APPEND(DMXConfigStringPtr, yyvsp[-1].string, yyvsp[0].string); yyval.string = yyvsp[-1].string; }
 break;
-#line 663 "parser.c"
+#line 641 "parser.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
@@ -709,13 +687,10 @@ to state %d\n", YYPREFIX, *yyssp, yystate);
     *++yyssp = yystate;
     *++yyvsp = yyval;
     goto yyloop;
-
 yyoverflow:
     yyerror("yacc stack overflow");
-
 yyabort:
     return (1);
-
 yyaccept:
     return (0);
 }
