@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2001-2003 Red Hat Inc., Durham, North Carolina.
  *
@@ -696,10 +695,11 @@ static void dmxConsoleComputeWidthHeight(myPrivate *priv,
 void dmxConsoleReInit(DevicePtr pDev)
 {
     GETPRIVFROMPDEV;
-    Display *dpy = priv->display;
+    Display *dpy;
 
     if (!priv || !priv->initialized) return;
-    
+    dpy = priv->display;
+
     dmxConsoleComputeWidthHeight(priv,
                                  &priv->width, &priv->height,
                                  &priv->xScale, &priv->yScale,
@@ -859,12 +859,17 @@ void dmxConsoleInit(DevicePtr pDev)
  * for pointers. */
 void dmxConsoleMouGetInfo(DevicePtr pDev, DMXLocalInitInfoPtr info)
 {
+    GETPRIVFROMPDEV;
+
     info->buttonClass      = 1;
     dmxCommonMouGetMap(pDev, info->map, &info->numButtons);
     info->valuatorClass    = 1;
     info->numRelAxes       = 2;
-    info->minval[0]        = 0;
-    info->maxval[0]        = 0;
+    info->minval[0] = 0;
+    info->minval[1] = 0;
+    /* max possible console window size: */
+    info->maxval[0] = DisplayWidth(priv->display, DefaultScreen(priv->display));
+    info->maxval[1] = DisplayHeight(priv->display, DefaultScreen(priv->display));
     info->res[0]           = 1;
     info->minres[0]        = 0;
     info->maxres[0]        = 1;
