@@ -1,3 +1,3 @@
 #define OPEQ MFB_OPEQ_WHITE
 #define MFBPOLYGLYPHBLT mfbPolyGlyphBltWhite
-#include "./mfbplygblt.c"
+#include "../../mfb/mfbplygblt.c"
